/* wsb.c (C) A.Zeuner 1997*/

#define WSB_DEBUG	0

#if defined (MODULE)
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/types.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/ioctl.h>
#include <asm/segment.h>
#include <asm/io.h>

#include "wsb.h"
#include "ptc.h"

#define WSB_MEMSIZE	(2*0x8000)

static char wsbname[]="wsb";
static unsigned pid=0;
static unsigned wsb_baseport=0x00;
static unsigned wsb_irq=0;
static unsigned short clRateLo=0;
static unsigned short clRateHi=0;


/*  inline functions and macros */
/*  These have to be changed for linux: */
#define outB(a,b) outb((b),wsb_baseport+(a))

#define outW(a,b) outw((b),wsb_baseport+(a))

#define inW(a)    (inw(wsb_baseport+(a)))

#define inB(a)    (inb(wsb_baseport+(a)))

/* These are identical */

#define setBaseMem(data) outW(W16_BASEMEM,(data))

#define setNrOfPoints(data) outW(W16_POINTNR,(data))

#define setPTC0(data)  setptc0(wsb_baseport+OFS_PIT,PTC_MODE2,(data))

#define setPTC1(data)  setptc1(wsb_baseport+OFS_PIT,PTC_MODE2,(data))

#define setPTC2(data)  setptc2(wsb_baseport+OFS_PIT,PTC_MODE1,(data))

#define setNrOfCycles(data) setPTC1(data)

#define setTriggerDelay(data) setPTC2(data)

#define setLoClockRate(data) \
   clRateLo=(data);          \
   outW(W16_CLOCKRATE,data)

#define setHiClockRate(data) \
   clRateHi=(data); \
   setPTC0(data)

#define resetPTC()  \
   setPTC0(0x0003); \
   setPTC1(0x0002); \
   setPTC2(0x0001)

#define getConfig() inW(RW16_CONFIG)

#define getConfigHi() inB(RW8_RESET)

#define getConfigLo() inB(RW8_MODE)

#define setConfig(data) outW(RW16_CONFIG,data)

#define setConfigLo(data) outB(RW16_CONFIG,((data) & (~SCALE)) )

#define setScale(data)    outB(RW16_CONFIG,((data) | SCALE))

#define setConfigHi(data) outB((RW16_CONFIG+1),data)

#define putData(data) outW(RW16_DATA,data)

#define readData() inW(RW16_DATA)

#define isRun()  (getConfigHi() & RUN_8)

#define setDelay(data) outW(W16_DELAYCYCLES,data)



void wsb_reset(void)
{
  int i;
setConfig(RESET);
setConfig(0);
setNrOfPoints(0);
setBaseMem(0);
resetPTC();
setLoClockRate(3);
setHiClockRate(1);
setConfig(DIR);
for (i=0;i<WSB_MEMSIZE/2;i++) {
   putData(0);      /* Memory der WSB 100 lschen */
}
setConfig(0);
setBaseMem(0);
}



static int wsb_lseek(struct inode* ino, struct file* f, 
		     off_t offs, int origin)
{
f->f_pos=0; 
#if (WSB_DEBUG)
printk("wsb_lseek 1: f->fpos=%i,offs: %i,mode: %i\n",f->f_pos,offs,origin);
#endif
return (0);
}



static int wsb_read(struct inode* ino, struct file* f,
		    char* buf, int count)
{
  unsigned short* p;
  int n;
#if (WSB_DEBUG)  
  printk("wsb: 0x%08x count: %i\n", buf,count);  
#endif  
  if (isRun())
    return -EINVAL;
  if (f->f_pos+count>=WSB_MEMSIZE) {
    return -EIO;
  }
  if (verify_area(VERIFY_WRITE,buf,count)) {
    return -EINVAL;
  }
  setBaseMem((unsigned short)f->f_pos/2);
  p=(unsigned short*)buf;
  for (n=0;n<count/2;n++) {
    put_user(readData(),p);
    ++p;
  } 
  /* f->f_pos += ( count & 0xfffe); */
  return (count & 0xfffe);
}

static int wsb_write(struct inode* ino, struct file* f,
		     const char* buf, int count)
{
  unsigned short* p;
  int n;
#if (WSB_DEBUG)  
  printk("wsb: 0x%08x count: %i\n", buf,count);  
#endif
  if (isRun())
    return -EINVAL;
  if (f->f_pos+count>=WSB_MEMSIZE) {
    return -EIO;
  }
  if (verify_area(VERIFY_READ,buf,count)) {
    return -EINVAL;
  }
  setBaseMem((unsigned short)f->f_pos/2);
  p=(unsigned short*)buf;
  for (n=0;n<count/2;n++) {
    putData(get_user(p));
    ++p;
  } 
  /* f->f_pos += (count & 0xfffe); */
  return (count & 0xfffe);
}

static 
int wsb_ioctl(struct inode* ino, struct file* f, 
              unsigned ioctlcmd, unsigned long arg)
{
  int rc=0;
  int type,cmd;
  unsigned lo,hi;

#if (WSB_DEBUG)
  printk("wsb: ioctl: 0x%08x type: 0x%08x cmd: 0x%08x\n",
	 ioctlcmd,_IOC_TYPE(ioctlcmd),_IOC_NR(ioctlcmd));
#endif

  type = _IOC_TYPE(ioctlcmd);
  if (type != WSB_IOCTL) 
    return -EINVAL;
  cmd  = _IOC_NR(ioctlcmd);

  switch (ioctlcmd) {
  case WSB_RESET:
    setConfig(RESET);
    setConfig(0);
    break;
  case WSB_SCALE_A12M:
    setScale(arg);
    break;
  case WSB_RUN:
    setConfigLo( STARTRUN | getConfigLo() );
    break;
  case WSB_STOP:
    setConfigLo( (~STARTRUN)  & getConfigLo() );
    break;
  case WSB_CONT:
    setConfigLo( getConfigLo() | CONT | REPT );
    break;
  case WSB_NOCONT:
    setConfigLo( getConfigLo() & ~CONT);
    break;
  case WSB_REPT:
    setNrOfCycles(arg);
    setConfigLo(getConfigLo() | REPT);
    break;
  case WSB_NOREPT:
    setConfigLo(getConfigLo() & ~REPT);
    break;
  case WSB_CLOCK:
    lo=arg & 0xffff;
    hi=(arg >>16) & 0xffff;
    if (lo<3) {
      rc = -EINVAL;  
      break;
    }
    if (hi==1) {
      setLoClockRate(lo);
      setConfigLo(getConfigLo() & ~(CSRC0 | CSRC1));
    } else {
      setLoClockRate(lo);
      setHiClockRate(hi);
      setConfigLo(getConfigLo() | CSRC0);
    }
    break;
  case WSB_BASEMEM:
    setBaseMem(arg);
    break;
  case WSB_POINTNR:
    setNrOfPoints(arg);
    break;
  case WSB_DELAY:
    setDelay(arg);
    break;
  case WSB_TRIGGERDELAY:
    setTriggerDelay(arg);
    break;
  case WSB_SETDIR:
    /* 1 increment 0 decrement */
    if (arg==0) {
      setConfigHi(getConfigHi() & ~DIR_8);
    } else {
      setConfigHi(getConfigHi() | DIR_8);
    } /* endif */
    break;
  case WSB_GETCONFIG:
    if (verify_area(VERIFY_WRITE,(const void*) arg,1*sizeof(short))) {
      rc =  -EINVAL;
      break;
    } /* endif */
    put_user(getConfig(),(unsigned short*)arg);
    break;
  case WSB_RTRIG:
    if (arg!=0) {
      setConfigHi(getConfigHi() | RTRIG_8);
    } else {
      setConfigHi(getConfigHi() & ~RTRIG_8);
    } /* endif */
    break;
  default:
    rc= -EINVAL;
    break;
  }
  return rc;
}

static int wsb_open(struct inode* ino, struct file* f)
{
  if (pid!=0) {
    return -EBUSY;
  }
  pid=current->pid;
#if (WSB_DEBUG)
  printk("wsb_open: pid=%i\n",pid);
#endif
  MOD_INC_USE_COUNT;
return 0;
}

static void wsb_release(struct inode* ino, struct file* f)
{
  pid=0;
#if (WSB_DEBUG)
  printk("wsb_release:\n");
#endif
  wsb_reset();
  MOD_DEC_USE_COUNT;
} 
        

/* Data for this device */


static struct file_operations wsb_fops= {	
	wsb_lseek,	/* lseek */		
	wsb_read,	/* read  */		
	wsb_write,	/* write */		
	NULL,		/* readdir */		
	NULL,		/* select */		
	wsb_ioctl,	/* ioctl */		
	NULL,		/* mmap */		
	wsb_open,	/* open */		
	wsb_release	/* release */		
};


#ifdef MODULE
#define wsb_init init_module



int wsb[2]={0,0};

/* Now we can load wsb.o with: insmod wsb.o wsb=0xf000,0x0f */ 

void cleanup_module(void)
{
  if (unregister_chrdev(WSB_MAJOR,wsbname))
    printk("wsb: Oops could not unregister\n");
  release_region(wsb_baseport,WSB_IOSIZE);
printk("wsb cleanup\n");
}

#endif

int wsb_init()
{
  if (wsb[0]==0) {
    printk("wsb: no ioport given\n");
    return 1;
  }
  if (check_region(wsb[0],WSB_IOSIZE)<0) {
    printk("wsb: ioports at 0x%04x already claimed\n",wsb[0]);
    return 1;
  }
  if (register_chrdev(WSB_MAJOR,wsbname,&wsb_fops)) {
    printk("wsb: could not register!\n");
    return 1;
  }
  wsb_baseport=wsb[0];
  wsb_irq=wsb[1];
  request_region(wsb_baseport,WSB_IOSIZE,wsbname);  
  printk("wsb: io: 0x%04x irq: 0x%04x\n",wsb[0],wsb[1]);
return 0;
}










