/************************************************************* */
/* wsbaccess.c  (C) A. Zeuner 1997                             */
/*           Subsystem fr WSB                                 */
/************************************************************* */

#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include "wsb.h"


/************************************************************* */
/* Globale Variablen */
/************************************************************* */

static int wsb=0;
static const double maxFreq=10.0e6;
static const double wsbFreq=40.0e6;
static const char* drvname="/dev/wsb";


// __export WsbOpen
int  WsbOpen
       (void)
{
wsb=open(drvname,O_RDWR);
return wsb;
}


// __export WsbClose
int  WsbClose(void)
{
if (wsb>0) {
	close(wsb);
     	wsb=0;
}
return (wsb);
}

// __export WsbInit
int  WsbInit(void)
{
if (wsb<=0) return WsbOpen();
else return wsb;
}

// __EXPORT WsbExit
int  WsbExit(void)
{
return WsbClose();
}

// __EXPORT WsbReset
int  WsbReset(void)
{
return ioctl(wsb,WSB_RESET,0);
}

// __export WsbScaleA12M
int  WsbScaleA12M(unsigned a12scale)
{
  return ioctl(wsb,WSB_SCALE_A12M,a12scale);
}



// __export WsbStop
int  WsbStop()
{
return ioctl(wsb,WSB_STOP,0);
}

// __export WsbRun
int  WsbRun(void)
{
return ioctl(wsb,WSB_RUN,0);
}

// __export WsbRunMode
int  WsbRunMode( unsigned flag)
   /* 0 Stop 1 Run */
{
  if (flag) {
    return WsbRun();
  }       
  return (WsbStop());
}


// __export WsbCont
int  WsbCont(unsigned flag)
   /* 1 kontinuierlich, 0 nur feste Anzahl */
{
  if (flag) {
    return ioctl(wsb,WSB_CONT,0);
  }
  return ioctl(wsb,WSB_NOCONT);
}

// __export WsbRept
int  WsbRept(unsigned flag,unsigned rept)
{
  if (flag) {
    if (rept<2) {
      return -1;
    } /* endif */
    return ioctl(wsb,WSB_REPT,rept);
  }
  return ioctl(wsb,WSB_NOREPT);
}

// __export WsbWriteClock
int  WsbWriteClock(unsigned lo, unsigned hi)
{
   unsigned t;
t= (lo & 0xffff) | ( hi << 16);
return ioctl(wsb,WSB_CLOCK,t);
}

// __export WsbReadClock
int  WsbReadClock(unsigned short*  tmr0, unsigned short* tmr1)
{
   int rc;
   unsigned t;
   if (tmr0==NULL || tmr1==0 ) {
     return (-1);
   } /* endif */
   rc= ioctl(wsb,WSB_CLOCK_R,(char*)&t);
   if (rc==0) {
     *tmr0=t & 0xffff;
     *tmr1=(t >>16);
   }
   return (rc);
}


// __export WsbBaseMem
int  WsbBaseMem(unsigned base)
{
  if (base>0x7FFF) {
    return (-1);
  } /* endif */
  return ioctl(wsb,WSB_BASEMEM,base);
}

// __export WsbPointNr
int  WsbPointNr(unsigned pointNr)
{

   if (pointNr>0x7FFF) {
     return (-1);
   } /* endif */
   return ioctl(wsb,WSB_POINTNR,pointNr);
}

// __export WsbDelay
int  WsbDelay(unsigned delay)
{
  if (!delay) {
    return (-1);
  } /* endif */
  return ioctl(wsb,WSB_DELAY);
}


// __export WsbTriggerDelay
int  WsbTriggerDelay(unsigned tdelay)
{
  if (!tdelay) {
    return (-1);
  } /* endif */
  return ioctl(wsb,WSB_TRIGGERDELAY,tdelay);
}


// __export WsbSetDir
int  WsbSetDir(unsigned direction)
   /* 0 decrement , 1 increment */
{
return ioctl(wsb,WSB_SETDIR,direction);
}

// __export WsbSetReTrigger
int  WsbSetReTrigger(unsigned flag)
   /* 0 off , 1 on */
{
return ioctl(wsb,WSB_RTRIG,flag);
}


// __export WsbGetConfig
int  WsbGetConfig(unsigned short * cfg)
{
return ioctl(wsb,WSB_GETCONFIG,cfg);
} 



// __export WsbGetFreqFromTimer
int  WsbGetFreqFromTimer
   (unsigned short tmr0, unsigned short tmr1, double* pfreq)
{
if (tmr0<3 || tmr1==0) {
   return (-1);
} /* endif */
*pfreq=wsbFreq/((tmr0+1)*tmr1);
return (0);
}


// __export WsbGetTimerFromFreq
int  WsbGetTimerFromFreq
      (double freq, unsigned short* t0, unsigned short* t1, double* rfreq)
{
/* f=wsbFreq/((t0+1)*t1) */
/* t0+1*t1=wsbFreq/f     */
   unsigned short lo,hi;
if (freq>maxFreq || freq<wsbFreq/((double)0x10000*(double)0xFFFF)||
    t0==NULL || t1==NULL || rfreq==NULL) {
   return (-1);
} /* endif */
freq=wsbFreq/freq;
hi=1;
while (freq/hi>0xFFFE) {
   hi++;
} /* endwhile */
lo=((freq)/hi)-1;
if (lo<3) {
   return (-1);
} /* endif */
*t0=lo;
*t1=hi;
*rfreq=wsbFreq/(lo+1)/hi;
return (0);
}



int WsbWriteData(unsigned base, unsigned count, 
		 unsigned dir, const unsigned short* data)
{
  if (base > 0x7fff)
    return -1;
  if (count >0x8000) 
    return -1;
  if (WsbBaseMem(base)<0)
    return -1;
  if (WsbSetDir(dir)<0)
    return -1;
  return write(wsb,(void*)data,count*2);
}

int WsbReadData(unsigned base, unsigned count,
		unsigned dir, unsigned short* data)
{
  if (base > 0x7fff)
    return -1;
  if (count >0x8000) 
    return -1;
  if (WsbBaseMem(base)<0)
    return -1;
  if (WsbSetDir(dir)<0)
    return -1;
  return read(wsb,(void*)data,count*2);
}




// __export WsbStartContOutput
int  WsbStartContOutput (unsigned  base, unsigned count, 
			 unsigned  dir, unsigned short* data, double freq)
{
   int rc;
   unsigned short t[2];
   rc=WsbWriteData(base,count,dir,data);
   if (rc<0) return (rc);
   rc=WsbSetDir(dir);
   if (rc<0) return (rc);
   freq*=count;
   rc=WsbGetTimerFromFreq(freq,t,t+1,&freq);
   if (rc<0) return (rc);
   rc=WsbWriteClock(t[0],t[1]);
   if (rc<0) return (rc);
   rc=WsbCont(1);
   if (rc<0) return (rc);
   rc=WsbBaseMem(base);
   if (rc<0) return (rc);
   rc=WsbSetDir(dir);
   if (rc<0) return (rc);
   rc=WsbPointNr(count-1);
   if (rc<0) return (rc);
   rc=WsbSetReTrigger(1);
   if (rc<0) return (rc);
   rc=WsbRun();
   return (rc);
}


// __export WsbPrepareA12MOutput
int  WsbPrepareA12MOutput
      (unsigned size, short* src, unsigned short* dst)
{
while (size--) {
   *dst= (*src+0x7FF) & 0xFFF;
   dst++;src++;
} /* endwhile */
return (0);
}

// __export WsbPrepareBipolarA12MOutput
int  WsbPrepareBipolarA12MOutput
      (unsigned size, short* src, unsigned short* dst)
{
while (size--) {
   *dst= (*src+0x7FF) & 0xFFF;
   dst++;src++;
} /* endwhile */
return (0);
}

// __export WsbPrepareMonopolarA12MOutput
int  WsbPrepareMonopolarA12MOutput
      (unsigned size, short* src, unsigned short* dst)
{
while (size--) {
   *dst = (*src & 0xFFF);
   dst++;src++;
} /* endwhile */
return (0);
}





